#include "buffer.h"

char* Buffer::retval;
int   Buffer::retvalsz;

void Buffer::Init()
{
	retval = (char*)malloc(32);
	retvalsz = 32;
}

void Buffer::Clean()
{
	free(retval);
}

Buffer::Buffer()
{
	data = (char*)malloc(32);
	count = 0;
	realsize = 32;
	readpos = 0;
	writepos = 0;
}

Buffer::~Buffer()
{
	if (data != NULL)
	{
		free(data);
		data = NULL;
	}
}

void Buffer::expand(int size)
{
	if ((writepos + size) >= realsize)
	{
		realsize = writepos + size + 32;
		data = (char*)realloc(data, realsize);
	}
}

void Buffer::write(void* src, int size)
{
	expand(size);
	memcpy(data + writepos, src, size);
	writepos += size;
	if (writepos > count)
		count = writepos;
}

void Buffer::read(void* dst, int size)
{
	if ((readpos + size) > count)
		size = count - readpos;
	if (size <= 0)
		return;
	memcpy(dst, data + readpos, size);
	readpos += size;
}

void Buffer::writebyte(unsigned char a)
{
	write(&a, 1);
}

void Buffer::writeshort(short a)
{
	write(&a, 2);
}

void Buffer::writeushort(unsigned short a)
{
	write(&a, 2);
}

void Buffer::writeint(int a)
{
	write(&a, 4);
}

void Buffer::writeuint(unsigned int a)
{
	write(&a, 4);
}

void Buffer::writefloat(float a)
{
	write(&a, 4);
}

void Buffer::writedouble(double a)
{
	write(&a, 8);
}

void Buffer::writechars(char* str, int len)
{
	write(str, len);
}

unsigned char Buffer::readbyte()
{
	unsigned char d;
	read(&d, 1);
	return d;
}

short Buffer::readshort()
{
	short d;
	read(&d, 2);
	return d;
}

unsigned short Buffer::readushort()
{
	unsigned short d;
	read(&d, 2);
	return d;
}

int Buffer::readint()
{
	int d;
	read(&d, 4);
	return d;
}

unsigned int Buffer::readuint()
{
	unsigned int d;
	read(&d, 4);
	return d;
}

float Buffer::readfloat()
{
	float d;
	read(&d, 4);
	return d;
}

double Buffer::readdouble()
{
	double d;
	read(&d, 8);
	return d;
}

char* Buffer::readchars(int len)
{
	if ((len + 1) > retvalsz)
	{
		retvalsz = len + 1;
		retval = (char*)realloc(retval, retvalsz);
	}
	read(retval, len);
	retval[len] = '\0';
	return retval;
}

char* Buffer::readstring()
{
	if (readpos == count)
		return NULL;
	int len;
	for(int i = readpos; i < count; i++)
	{
		if (data[i] == '\0')
		{
			len = i - readpos + 1;
			if (len > retvalsz)
			{
				retvalsz = len;
				retval = (char*)realloc(retval, retvalsz);
			}
			read(retval, len);
			return retval;
		}
	}
	len = count - readpos;
	if ((len + 1) > retvalsz)
	{
		retvalsz = len + 1;
		retval = (char*)realloc(retval, retvalsz);
	}
	read(retval, len);
	retval[len] = 0;
	return retval;
}

char* Buffer::readsep(char* sep)
{
	if (readpos == count)
		return NULL;
	int len, j;
	int slen = (int)strlen(sep);
	int k = count - slen + 1;
	for (int i = readpos; i < k; i++)
	{
		for (j = 0; j < slen; j++)
		{
			if (data[i + j] != sep[j])
				break;
		}
		if (j == slen)
		{
			len = i - readpos;
			if ((len + 1) > retvalsz)
			{
				retvalsz = len + 1;
				retval = (char*)realloc(retval, retvalsz);
			}
			read(retval, len);
			retval[len] = '\0';
			readpos += slen;
			return retval;
		}
	}
	len = count - readpos;
	if ((len + 1) > retvalsz)
	{
		retvalsz = len + 1;
		retval = (char*)realloc(retval, retvalsz);
	}
	read(retval, len);
	retval[len] = '\0';
	return retval;
}


void Buffer::clear()
{
	count = 0;
	readpos = 0;
	writepos = 0;
}

void Buffer::setwrite(int pos, bool rel)
{
	if (rel)
		pos += writepos;
	else if (pos < 0)
		pos += count;

	if (pos < 0)
		writepos = 0;
	else if (pos > count)
		writepos = count;
	else
		writepos = pos;
}

void Buffer::setread(int pos, bool rel)
{
	if (rel)
		pos += readpos;
	else if (pos < 0)
		pos += count;

	if (pos < 0)
		readpos = 0;
	else if (pos > count)
		readpos = count;
	else
		readpos = pos;
}