#include "file.h"

File::File(char* filename, int mode)
{
	DWORD access;
	if (mode == 0)
		access = GENERIC_READ;
	else if (mode == 1)
		access = GENERIC_WRITE;
	else
		access = GENERIC_READ | GENERIC_WRITE;
	hwnd = CreateFileA(filename, access, FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hwnd != NULL)
	{
		size = GetFileSize(hwnd, NULL);
		ptr = 0;
	}
}

File::~File()
{
	if (hwnd == NULL)
		return;
	CloseHandle(hwnd);
}

int File::Read(char* str, int len)
{
	if (hwnd == NULL)
		return -1;
	if (size <= 0)
		return -1;
	DWORD rsize;
	ReadFile(hwnd, str, len, &rsize, NULL);
	ptr += rsize;
	return (int)rsize;
}

int File::Write(char* str, int len)
{
	if (hwnd == NULL)
		return -1;
	if (size <= 0)
		return -1;
	DWORD wsize;
	WriteFile(hwnd, str, len, &wsize, NULL);
	ptr += wsize;
	if (ptr >= size)
		size = ptr;
	return (int)wsize;
}

int File::GetPos()
{
	if (hwnd == NULL)
		return -1;
	return ptr;
}

int File::SetPos(int pos, bool rel)
{
	if (hwnd == NULL)
		return -1;
	if (rel)
		pos += ptr;
	else if (pos < 0)
		pos += size;

	if (pos < 0)
		pos = 0;
	else if (pos >= size)
		pos = size;

	ptr = SetFilePointer(hwnd, pos, NULL, FILE_BEGIN);
	return ptr;
}

int File::Size()
{
	if (hwnd == NULL)
		return 0;
	return size;
}

bool File::Close()
{
	HANDLE h = hwnd;
	if (h == NULL)
		return false;
	hwnd = NULL;
	if (CloseHandle(h))
		return true;
	return false;
}