#include "hash.h"

char Hash::LuhnD[20];
char Hash::VerDataD[100];
char Hash::VerDataP[80];
UINT32* Hash::Ret;
char* Hash::Digest;

#ifndef ROL32
	#define ROL32(a,c) (((a) << (c)) | ((a) >> (32-(c))))
#endif

#ifndef ROR32
	#define ROR32(a,c) (((a) >> (c)) | ((a) << (32-(c))))
#endif

void Hash::Init()
{
	char ld[20] =		{	0,1,2,3,4,5,6,7,8,9,
							0,2,4,6,8,1,3,5,7,9};
	char vd[100] = 		{	0,1,2,3,4,5,6,7,8,9,
							1,2,3,4,0,6,7,8,9,5,
							2,3,4,0,1,7,8,9,5,6,
							3,4,0,1,2,8,9,5,6,7,
							4,0,1,2,3,9,5,6,7,8,
							5,9,8,7,6,0,4,3,2,1,
							6,5,9,8,7,1,0,4,3,2,
							7,6,5,9,8,2,1,0,4,3,
							8,7,6,5,9,3,2,1,0,4,
							9,8,7,6,5,4,3,2,1,0};
	char vp[80] = 		{	0,1,2,3,4,5,6,7,8,9,
							1,5,7,6,2,8,3,0,9,4,
							5,8,0,3,7,9,6,1,4,2,
							8,9,1,6,0,4,3,5,2,7,
							9,4,5,3,1,2,6,8,7,0,
							4,2,8,6,5,7,3,9,0,1,
							2,7,9,3,8,0,6,4,1,5,
							7,0,4,6,9,1,3,2,5,8};
	memcpy(&LuhnD, &ld, 20);
	memcpy(&VerDataD, &vd, 100);
	memcpy(&VerDataP, &vp, 80);
	Ret = (UINT32*)malloc(32);
	Digest = (char*)malloc(65);
}

void Hash::Clean()
{
	free(Ret);
	free(Digest);
}

int Hash::AddHash(char* str, int size, int block, int mod)
{
	long j = 0;
	long k;
	int i;

	if (size == 0)
		size = (int)strlen(str);
	switch(block)
	{
		case 1:
		default:
			for (i = 0; i < size; i++)
				j = (j + str[i]) % mod;
			return j;
		case 2:
			for (i = 0; i < size; i += 2)
			{
				k = str[i+1];
				j = (j + (k<<8)|(str[i])) % mod;
			}
			return j;
		case 4:
			for (i = 0; i < size; i += 4)
			{
				k = str[i+3];
				k = (k<<8)|(str[i+2]);
				k = (k<<8)|(str[i+1]);
				j = (j + (k<<8)|(str[i])) % mod;
			}
			return j;
	}
}

int Hash::XorHash(char* str, int size, int block)
{
	long j = 0;
	long k;
	int i;

	if (size == 0)
		size = (int)strlen(str);
	switch(block)
	{
		case 1:
		default:
			for (i = 0; i < size; i++)
				j ^= str[i];
			return j;
		case 2:
			for (i = 0; i < size; i += 2)
			{
				k = str[i+1];
				j ^= (k<<8)|(str[i]);
			}
			return j;
		case 4:
			for (i = 0; i < size; i += 4)
			{
				k = str[i+3];
				k = (k<<8)|(str[i+2]);
				k = (k<<8)|(str[i+1]);
				j ^= (k<<8)|(str[i]);
			}
			return j;
	}
}

#define SHA1_R1(a,b,c,d,e,t1,t2,i) {(t1) = ROL32(a,5) + (((b)&(c))|(~(b)&d))            + (e) + (i) + (UINT32)0x5A827999; (t2) = ROR32(b,2);}
#define SHA1_R2(a,b,c,d,e,t1,t2,i) {(t1) = ROL32(a,5) + ((b)^(c)^(d))                   + (e) + (i) + (UINT32)0x6ED9EBA1; (t2) = ROR32(b,2);}
#define SHA1_R3(a,b,c,d,e,t1,t2,i) {(t1) = ROL32(a,5) + (((b)&(c))|((b)&(d))|((c)&(d))) + (e) + (i) + (UINT32)0x8F1BBCDC; (t2) = ROR32(b,2);}
#define SHA1_R4(a,b,c,d,e,t1,t2,i) {(t1) = ROL32(a,5) + ((b)^(c)^(d))                   + (e) + (i) + (UINT32)0xCA62C1D6; (t2) = ROR32(b,2);}

char* Hash::SHA1(char* str, int size)
{
	UINT32 a, b, c, d, e, v, w;
	UINT32* t;
	UINT32 u[80];
	int i, j, rs;

	Ret[0] = 0x67452301;
	Ret[1] = 0xEFCDAB89;
	Ret[2] = 0x98BADCFE;
	Ret[3] = 0x10325476;
	Ret[4] = 0xC3D2E1F0;
	if (size == 0)
		size = (int)strlen(str);
	rs = (size + 72) & 0xFFFFFFC0;
	t = new UINT32[rs>>2];
	memcpy(t, str, size);
	((char*)t)[size] = 128;
	for(i = size + 1; i < (rs - 8); i++)
		((char*)t)[i] = 0;
	rs >>= 2;
	t[rs-2] = (size >> 29) & 0xFFFFFFFF;
	t[rs-1] = (size << 3)  & 0xFFFFFFFF;
	for(i = 0; i < rs; i += 16)
	{
		memcpy(&u, t + i, 64);
		for(j = 16; j < 80; j++)
		{
			u[j] = u[j - 3] ^ u[j - 8] ^ u[j - 14] ^ u[j - 16];
			u[j] = ROL32(u[j],1);
		}

		a = Ret[0];
		b = Ret[1];
		c = Ret[2];
		d = Ret[3];
		e = Ret[4];

		SHA1_R1(a,b,c,d,e, v,w, u[0]); // Round 1
		SHA1_R1(v,a,w,c,d, b,e, u[1]);
		SHA1_R1(b,v,e,w,c, a,d, u[2]);
		SHA1_R1(a,b,d,e,w, v,c, u[3]);
		SHA1_R1(v,a,c,d,e, b,w, u[4]);
		SHA1_R1(b,v,w,c,d, a,e, u[5]);
		SHA1_R1(a,b,e,w,c, v,d, u[6]);
		SHA1_R1(v,a,d,e,w, b,c, u[7]);
		SHA1_R1(b,v,c,d,e, a,w, u[8]);
		SHA1_R1(a,b,w,c,d, v,e, u[9]);
		SHA1_R1(v,a,e,w,c, b,d, u[10]);
		SHA1_R1(b,v,d,e,w, a,c, u[11]);

		SHA1_R1(a,b,c,d,e, v,w, u[12]);
		SHA1_R1(v,a,w,c,d, b,e, u[13]);
		SHA1_R1(b,v,e,w,c, a,d, u[14]);
		SHA1_R1(a,b,d,e,w, v,c, u[15]);
		SHA1_R1(v,a,c,d,e, b,w, u[16]);
		SHA1_R1(b,v,w,c,d, a,e, u[17]);
		SHA1_R1(a,b,e,w,c, v,d, u[18]);
		SHA1_R1(v,a,d,e,w, b,c, u[19]);
		SHA1_R2(b,v,c,d,e, a,w, u[20]); //Round 2
		SHA1_R2(a,b,w,c,d, v,e, u[21]);
		SHA1_R2(v,a,e,w,c, b,d, u[22]);
		SHA1_R2(b,v,d,e,w, a,c, u[23]);

		SHA1_R2(a,b,c,d,e, v,w, u[24]);
		SHA1_R2(v,a,w,c,d, b,e, u[25]);
		SHA1_R2(b,v,e,w,c, a,d, u[26]);
		SHA1_R2(a,b,d,e,w, v,c, u[27]);
		SHA1_R2(v,a,c,d,e, b,w, u[28]);
		SHA1_R2(b,v,w,c,d, a,e, u[29]);
		SHA1_R2(a,b,e,w,c, v,d, u[30]);
		SHA1_R2(v,a,d,e,w, b,c, u[31]);
		SHA1_R2(b,v,c,d,e, a,w, u[32]);
		SHA1_R2(a,b,w,c,d, v,e, u[33]);
		SHA1_R2(v,a,e,w,c, b,d, u[34]);
		SHA1_R2(b,v,d,e,w, a,c, u[35]);

		SHA1_R2(a,b,c,d,e, v,w, u[36]);
		SHA1_R2(v,a,w,c,d, b,e, u[37]);
		SHA1_R2(b,v,e,w,c, a,d, u[38]);
		SHA1_R2(a,b,d,e,w, v,c, u[39]);
		SHA1_R3(v,a,c,d,e, b,w, u[40]); //Round 3
		SHA1_R3(b,v,w,c,d, a,e, u[41]);
		SHA1_R3(a,b,e,w,c, v,d, u[42]);
		SHA1_R3(v,a,d,e,w, b,c, u[43]);
		SHA1_R3(b,v,c,d,e, a,w, u[44]);
		SHA1_R3(a,b,w,c,d, v,e, u[45]);
		SHA1_R3(v,a,e,w,c, b,d, u[46]);
		SHA1_R3(b,v,d,e,w, a,c, u[47]);

		SHA1_R3(a,b,c,d,e, v,w, u[48]);
		SHA1_R3(v,a,w,c,d, b,e, u[49]);
		SHA1_R3(b,v,e,w,c, a,d, u[50]);
		SHA1_R3(a,b,d,e,w, v,c, u[51]);
		SHA1_R3(v,a,c,d,e, b,w, u[52]);
		SHA1_R3(b,v,w,c,d, a,e, u[53]);
		SHA1_R3(a,b,e,w,c, v,d, u[54]);
		SHA1_R3(v,a,d,e,w, b,c, u[55]);
		SHA1_R3(b,v,c,d,e, a,w, u[56]);
		SHA1_R3(a,b,w,c,d, v,e, u[57]);
		SHA1_R3(v,a,e,w,c, b,d, u[58]);
		SHA1_R3(b,v,d,e,w, a,c, u[59]);

		SHA1_R4(a,b,c,d,e, v,w, u[60]); // Round 4
		SHA1_R4(v,a,w,c,d, b,e, u[61]);
		SHA1_R4(b,v,e,w,c, a,d, u[62]);
		SHA1_R4(a,b,d,e,w, v,c, u[63]);
		SHA1_R4(v,a,c,d,e, b,w, u[64]);
		SHA1_R4(b,v,w,c,d, a,e, u[65]);
		SHA1_R4(a,b,e,w,c, v,d, u[66]);
		SHA1_R4(v,a,d,e,w, b,c, u[67]);
		SHA1_R4(b,v,c,d,e, a,w, u[68]);
		SHA1_R4(a,b,w,c,d, v,e, u[69]);
		SHA1_R4(v,a,e,w,c, b,d, u[70]);
		SHA1_R4(b,v,d,e,w, a,c, u[71]);

		SHA1_R4(a,b,c,d,e, v,w, u[72]);
		SHA1_R4(v,a,w,c,d, b,e, u[73]);
		SHA1_R4(b,v,e,w,c, a,d, u[74]);
		SHA1_R4(a,b,d,e,w, v,c, u[75]);
		SHA1_R4(v,a,c,d,e, b,w, u[76]);
		SHA1_R4(b,v,w,c,d, a,e, u[77]);
		SHA1_R4(a,b,e,w,c, v,d, u[78]);
		SHA1_R4(v,a,d,e,w, b,c, u[79]);

		Ret[0] += b;
		Ret[1] += v;
		Ret[2] += c;
		Ret[3] += d;
		Ret[4] += e;
	}
	delete(t);
	for (i = 0; i < 20; i++)
		sprintf(Digest + i * 2, "%02X", *((char*)Ret + i));
	Digest[40] = '\0';
	return &Digest[0];
}

#define SHA2_R(a,b,c,d,e,f,g,h,t1,t2,i,k) {(t2) = (h) + (ROR32(e,6)^ROR32(e,11)^ROL32(e,7)) + (((e)&(f))^((~(e))&(g))) + (k) + (i); (t1) = (ROR32(a,2)^ROR32(a,13)^ROL32(a,10)) + (((a)&(b))^((a)&(c))^((b)&(c))) + (t2); (t2) += (d);}

char* Hash::SHA2(char* str, int size)
{
	UINT32 a, b, c, d, e, f, g, h, v, w;
	UINT32* t;
	UINT32 u[64];
	int i, j, rs;

	Ret[0] = 0x6A09E667;
	Ret[1] = 0xBB67AE85;
	Ret[2] = 0x3C6EF372;
	Ret[3] = 0xA54FF53A;
	Ret[4] = 0x510E527F;
	Ret[5] = 0x9B05688C;
	Ret[6] = 0x1F83D9AB;
	Ret[7] = 0x5BE0CD19;
	if (size == 0)
		size = (int)strlen(str);
	rs = (size + 72) & 0xFFFFFFC0;
	t = new UINT32[rs>>2];
	memcpy(t, str, size);
	((char*)t)[size] = 128;
	for(i = size + 1; i < (rs - 8); i++)
		((char*)t)[i] = 0;
	rs >>= 2;
	t[rs-2] = (size >> 29) & 0xFFFFFFFF;
	t[rs-1] = (size << 3)  & 0xFFFFFFFF;
	for(i = 0; i < rs; i += 16)
	{
		memcpy(&u, t + i, 64);
		for(j = 16; j < 64; j++)
		{
			u[i] = u[i-16]
				+ (ROR32(u[i-15],7) ^ ROL32(u[i-15],14) ^ (u[i-15] >> 3))
				+ u[i-7]
				+ (ROL32(u[i-2],15) ^ ROL32(u[i-2],13) ^ (u[i-2] >> 10));
		}

		a = Ret[0];
		b = Ret[1];
		c = Ret[2];
		d = Ret[3];
		e = Ret[4];
		f = Ret[5];
		g = Ret[6];
		h = Ret[7];

		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[0], 0x428A2F98);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[1], 0x71374491);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[2], 0xB5C0FBCF);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[3], 0xE9B5DBA5);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[4], 0x3956C25B);
		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[5], 0x59F111F1);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[6], 0x923F82A4);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[7], 0xAB1C5ED5);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[8], 0xD807AA98);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[9], 0x12835B01);

		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[10], 0x243185BE);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[11], 0x550C7DC3);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[12], 0x72BE5D74);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[13], 0x80DEB1FE);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[14], 0x9BDC06A7);
		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[15], 0xC19BF174);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[16], 0xE49B69C1);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[17], 0xEFBE4786);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[18], 0x0FC19DC6);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[19], 0x240CA1CC);

		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[20], 0x2DE92C6F);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[21], 0x4A7484AA);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[22], 0x5CB0A9DC);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[23], 0x76F988DA);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[24], 0x983E5152);
		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[25], 0xA831C66D);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[26], 0xB00327C8);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[27], 0xBF597FC7);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[28], 0xC6E00BF3);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[29], 0xD5A79147);

		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[30], 0x06CA6351);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[31], 0x14292967);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[32], 0x27B70A85);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[33], 0x2E1B2138);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[34], 0x4D2C6DFC);
		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[35], 0x53380D13);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[36], 0x650A7354);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[37], 0x766A0ABB);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[38], 0x81C2C92E);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[39], 0x92722C85);

		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[40], 0xA2BFE8A1);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[41], 0xA81A664B);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[42], 0xC24B8B70);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[43], 0xC76C51A3);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[44], 0xD192E819);
		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[45], 0xD6990624);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[46], 0xF40E3585);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[47], 0x106AA070);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[48], 0x19A4C116);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[49], 0x1E376C08);

		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[50], 0x2748774C);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[51], 0x34B0BCB5);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[52], 0x391C0CB3);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[53], 0x4ED8AA4A);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[54], 0x5B9CCA4F);
		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[55], 0x682E6FF3);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[56], 0x748F82EE);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[57], 0x78A5636F);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[58], 0x84C87814);
		SHA2_R(b,c,d,v,f,g,h,w, a,b, u[59], 0x8CC70208);

		SHA2_R(a,b,c,d,e,f,g,h, v,w, u[60], 0x90BEFFFA);
		SHA2_R(v,a,b,c,w,e,f,g, d,h, u[61], 0xA4506CEB);
		SHA2_R(d,v,a,b,h,w,e,f, c,g, u[62], 0xBEF9A3F7);
		SHA2_R(c,d,v,a,g,h,w,e, b,f, u[63], 0xC67178F2);

		Ret[0] += b;
		Ret[1] += c;
		Ret[2] += d;
		Ret[3] += v;
		Ret[4] += f;
		Ret[5] += g;
		Ret[6] += h;
		Ret[7] += w;
	}
	delete(t);
	for (i = 0; i < 32; i++)
		sprintf(Digest + i * 2, "%02X", *((char*)Ret + i));
	Digest[64] = '\0';
	return &Digest[0];
}

#define MD5_R1(a,b,c,d,t,r,i,k) {(t) = a + (((b)&(c))|((~(b))&(d))) + (k) + (i); (t) = (b) + ROL32(t,r);}
#define MD5_R2(a,b,c,d,t,r,i,k) {(t) = a + (((d)&(b))|((~(d))&(c))) + (k) + (i); (t) = (b) + ROL32(t,r);}
#define MD5_R3(a,b,c,d,t,r,i,k) {(t) = a + ((b)^(c)^(d))            + (k) + (i); (t) = (b) + ROL32(t,r);}
#define MD5_R4(a,b,c,d,t,r,i,k) {(t) = a + ((c)^((b)|(~(d))))       + (k) + (i); (t) = (b) + ROL32(t,r);}

char* Hash::MD5(char* str, int size)
{
	UINT32 a, b, c, d, v;
	UINT32* t;
	UINT32 u[16];
	int i, rs;

	Ret[0] = 0x67452301;
	Ret[1] = 0xEFCDAB89;
	Ret[2] = 0x98BADCFE;
	Ret[3] = 0x10325476;
	if (size == 0)
		size = (int)strlen(str);
	rs = (size + 72) & 0xFFFFFFC0;
	t = new UINT32[rs>>2];
	memcpy(t, str, size);
	((char*)t)[size] = 128;
	for(i = size + 1; i < (rs - 8); i++)
		((char*)t)[i] = 0;
	rs >>= 2;
	t[rs-2] = (size >> 29) & 0xFFFFFFFF;
	t[rs-1] = (size << 3)  & 0xFFFFFFFF;
	for(i = 0; i < rs; i += 16)
	{
		memcpy(&u, t + i, 64);

		a = Ret[0];
		b = Ret[1];
		c = Ret[2];
		d = Ret[3];

		MD5_R1(a,b,c,d, v, 7,u[ 0], 0xD76AA478); //Round 1
		MD5_R1(d,v,b,c, a,12,u[ 1], 0xE8C7B756);
		MD5_R1(c,a,v,b, d,17,u[ 2], 0x242070DB);
		MD5_R1(b,d,a,v, c,22,u[ 3], 0xC1BDCEEE);
		MD5_R1(v,c,d,a, b, 7,u[ 4], 0xF57C0FAF);
		MD5_R1(a,b,c,d, v,12,u[ 5], 0x4787C62A);
		MD5_R1(d,v,b,c, a,17,u[ 6], 0xA8304613);
		MD5_R1(c,a,v,b, d,22,u[ 7], 0xFD469501);
		MD5_R1(b,d,a,v, c, 7,u[ 8], 0x698098D8);
		MD5_R1(v,c,d,a, b,12,u[ 9], 0x8B44F7AF);
		MD5_R1(a,b,c,d, v,17,u[10], 0xFFFF5BB1);
		MD5_R1(d,v,b,c, a,22,u[11], 0x895CD7BE);
		MD5_R1(c,a,v,b, d, 7,u[12], 0x6B901122);
		MD5_R1(b,d,a,v, c,12,u[13], 0xFD987193);
		MD5_R1(v,c,d,a, b,17,u[14], 0xA679438E);
		MD5_R1(a,b,c,d, v,22,u[15], 0x49B40821);

		MD5_R2(d,v,b,c, a, 5,u[ 1], 0xF61E2562); //Round 2
		MD5_R2(c,a,v,b, d, 9,u[ 6], 0xC040B340);
		MD5_R2(b,d,a,v, c,14,u[11], 0x265E5A51);
		MD5_R2(v,c,d,a, b,20,u[ 0], 0xE9B6C7AA);
		MD5_R2(a,b,c,d, v, 5,u[ 5], 0xD62F105D);
		MD5_R2(d,v,b,c, a, 9,u[10], 0x02441453);
		MD5_R2(c,a,v,b, d,14,u[15], 0xD8A1E681);
		MD5_R2(b,d,a,v, c,20,u[ 4], 0xE7D3FBC8);
		MD5_R2(v,c,d,a, b, 5,u[ 9], 0x21E1CDE6);
		MD5_R2(a,b,c,d, v, 9,u[14], 0xC33707D6);
		MD5_R2(d,v,b,c, a,14,u[ 3], 0xF4D50D87);
		MD5_R2(c,a,v,b, d,20,u[ 8], 0x455A14ED);
		MD5_R2(b,d,a,v, c, 5,u[13], 0xA9E3E905);
		MD5_R2(v,c,d,a, b, 9,u[ 2], 0xFCEFA3F8);
		MD5_R2(a,b,c,d, v,14,u[ 7], 0x676F02D9);
		MD5_R2(d,v,b,c, a,20,u[12], 0x8D2A4C8A);

		MD5_R3(c,a,v,b, d, 4,u[ 5], 0xFFFA3942); //Round 3
		MD5_R3(b,d,a,v, c,11,u[ 8], 0x8771F681);
		MD5_R3(v,c,d,a, b,16,u[11], 0x6D9D6122);
		MD5_R3(a,b,c,d, v,23,u[14], 0xFDE5380C);
		MD5_R3(d,v,b,c, a, 4,u[ 1], 0xA4BEEA44);
		MD5_R3(c,a,v,b, d,11,u[ 4], 0x4BDECFA9);
		MD5_R3(b,d,a,v, c,16,u[ 7], 0xF6BB4B60);
		MD5_R3(v,c,d,a, b,23,u[10], 0xBEBFBC70);
		MD5_R3(a,b,c,d, v, 4,u[13], 0x289B7EC6);
		MD5_R3(d,v,b,c, a,11,u[ 0], 0xEAA127FA);
		MD5_R3(c,a,v,b, d,16,u[ 3], 0xD4EF3085);
		MD5_R3(b,d,a,v, c,23,u[ 6], 0x04881D05);
		MD5_R3(v,c,d,a, b, 4,u[ 9], 0xD9D4D039);
		MD5_R3(a,b,c,d, v,11,u[12], 0xE6DB99E5);
		MD5_R3(d,v,b,c, a,16,u[15], 0x1FA27CF8);
		MD5_R3(c,a,v,b, d,23,u[ 2], 0xC4AC5665);

		MD5_R4(b,d,a,v, c, 6,u[ 0], 0xF4292244); //Round 4
		MD5_R4(v,c,d,a, b,10,u[ 7], 0x432AFF97);
		MD5_R4(a,b,c,d, v,15,u[14], 0xAB9423A7);
		MD5_R4(d,v,b,c, a,21,u[ 5], 0xFC93A039);
		MD5_R4(c,a,v,b, d, 6,u[12], 0x655B59C3);
		MD5_R4(b,d,a,v, c,10,u[ 3], 0x8F0CCC92);
		MD5_R4(v,c,d,a, b,15,u[10], 0xFFEFF47D);
		MD5_R4(a,b,c,d, v,21,u[ 1], 0x85845DD1);
		MD5_R4(d,v,b,c, a, 6,u[ 8], 0x6FA87E4F);
		MD5_R4(c,a,v,b, d,10,u[15], 0xFE2CE6E0);
		MD5_R4(b,d,a,v, c,15,u[ 6], 0xA3014314);
		MD5_R4(v,c,d,a, b,21,u[13], 0x4E0811A1);
		MD5_R4(a,b,c,d, v, 6,u[ 4], 0xF7537E82);
		MD5_R4(d,v,b,c, a,10,u[11], 0xBD3AF235);
		MD5_R4(c,a,v,b, d,15,u[ 2], 0x2AD7D2BB);
		MD5_R4(b,d,a,v, c,21,u[ 9], 0xEB86D391);

		Ret[0] += v;
		Ret[1] += c;
		Ret[2] += d;
		Ret[3] += a;
	}
	delete(t);
	for (i = 0; i < 16; i++)
		sprintf(Digest + i * 2, "%02X", *((char*)Ret + i));
	Digest[32] = '\0';
	return &Digest[0];
}

int Hash::Adler32(char* str, int size)
{
	int a = 1;
	int b = 0;
	int i;

	if (size == 0)
		size = (int)strlen(str);
	for (i = 0; i < size; i++)
	{
		a = (a + str[i]) % 65521;
		b = (b + a) % 65521;
	}
	return (b<<16)|a;
}

int Hash::Luhn(char* str, int size)
{
	int j = 0;
	int i, k;
	int m = 0;

	if (size == 0)
		size = (int)strlen(str);
	for (i = 0; i < size; i++)
	{
		k = str[i] - 48;
		if ((k >= 0) && (k < 10))
		{
			j = (j + LuhnD[m + k]) % 10;
			m = 10 - m;
		}
	}
	return j;
}

int Hash::Verhoeff(char* str, int size)
{
	int j = 0;
	int i, k;
	int m = 0;

	if (size == 0)
		size = (int)strlen(str);
	for (i = 0; i < size; i++)
	{
		k = str[i] - 48;
		if ((k >= 0) && (k <= 10))
		{
			j = VerDataD[j * 10 + VerDataP[m + k]];
			m = (m + 10) % 80;
		}
	}
	return j;
}