#include "main.h"

BOOL APIENTRY DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	int i;

	switch(fdwReason)
	{
		case DLL_PROCESS_ATTACH:
			Buffer::Init();
			Compress::Init();
			Hash::Init();
			buffers.items[0] = new Buffer();
			buffers.numfree--;
			Socket::Init();
			break;

		case DLL_PROCESS_DETACH:
			for(i = 0; i < sockets.size; i++)
				if(sockets.items[i] != NULL)
					delete (Socket*)sockets.items[i];
			free(sockets.items);
			for(i =  0; i < files.size; i++)
				if(files.items[i] != NULL)
					delete (File*)files.items[i];
			free(files.items);
			for(i = 0; i < buffers.size; i++)
				if(buffers.items[i] != NULL)
					delete (Buffer*)buffers.items[i];
			free(buffers.items);
			Buffer::Clean();
			Compress::Clean();
			Hash::Clean();
			Socket::Clean();
			break;
	}
	return TRUE;
}

DLLEXPORT double TCPConnect(char* Address, double Port)
{
	unsigned long ip = Socket::IP(Address);
	if(ip == 0)
		return -8;

	Socket* sock = new Socket();

	int er = sock->Connect(ip, (unsigned short)Port, 1);
	if(er < 0)
	{
		delete(sock);
		return er;
	}
	return sockets.Add(sock);
}

DLLEXPORT double TCPListen(double Port)
{
	Socket* sock = new Socket();

	int er = sock->Listen(1, (unsigned short)Port);
	if(er < 0)
	{
		delete(sock);
		return er;
	}
	return sockets.Add(sock);
}

DLLEXPORT double TCPAccept(double sID)
{
	Socket* sock = (Socket*)sockets[(int)sID];
	if(sock == NULL)
		return -9;

	Socket* sock2 = sock->Accept();
	if(sock2 == NULL)
		return -10;

	return sockets.Add(sock2);
}

DLLEXPORT double TCPIP(double sID)
{
	Socket* sock = (Socket*)sockets[(int)sID];
	if(sock == NULL)
		return -9;

	return sock->GetIP();
}

DLLEXPORT double TCPConnected(double sID)
{
	Socket* sock = (Socket*)sockets[(int)sID];
	if(sock == NULL)
		return -9;

	if(sock->Connected())
		return 1;
	return 0;
}

DLLEXPORT double UDPConnect(double Port)
{
	Socket* sock = new Socket();

	int er = sock->Listen(0, (unsigned short)Port);
	if(er < 0)
	{
		delete(sock);
		return er;
	}
	return sockets.Add(sock);
}

DLLEXPORT double SendMsg(double sID, char* Address, double Port, double bID)
{
	Socket* sock = (Socket*)sockets[(int)sID];
	if(sock == NULL)
		return -9;

	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	//UDP hack for backward compatibility
	if(sock->type == 0)
	{
		unsigned long ip = Socket::IP(Address);
		if(ip == 0)
			return -8;

		sock->other.sin_family = AF_INET;
		sock->other.sin_addr = *(IN_ADDR*)&ip;
		sock->other.sin_port = htons((unsigned short)Port);
	}
	return sock->Send(buff);
}

DLLEXPORT double ReceiveMsg(double sID, double Size, double bID)
{
	Socket* sock = (Socket*)sockets[(int)sID];
	if(sock == NULL)
		return -9;

	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	return sock->Recv(buff);
}

DLLEXPORT double PeekMsg(double sID, double Size, double bID)
{
	Socket* sock = (Socket*)sockets[(int)sID];
	if(sock == NULL)
		return -9;

	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	return sock->Peek(buff);
}

DLLEXPORT double CloseSocket(double sID)
{
	Socket* sock = (Socket*)sockets[(int)sID];
	if(sock == NULL)
		return -9;

	if(sock->id == 0)
		return -4;

	sockets.Remove((int)sID);
	delete(sock);
	return 0;
}

DLLEXPORT double LastError()
{
	return WSAGetLastError();
}

DLLEXPORT char* MyHost()
{
	static char buf[16];
	gethostname(buf, 16);
	return buf;
}

/* Provided by 39ster from the original 39dll
** I trust that he got it right because I am not
** 100% sure what is going on.
*/
DLLEXPORT double CompareIP(char* IP1, char* IP2)
{
	char *cp = 0, *mp = 0;
	while ((*IP1) && (*IP2 != '*'))
	{
		if((*IP2 != *IP1) && (*IP2 != '?'))
			return 0;
		IP2++;
		IP1++;
	}
	while (*IP1)
	{
		if (*IP2 == '*') 
		{
			if(!*++IP2) 
				return 1;
			mp = IP2;
			cp = IP1+1;
		}
		else if ((*IP2 == *IP1) || (*IP2 == '?')) 
		{
			IP2++;
			IP1++;
		}
		else 
		{
			IP2 = mp;
			IP1 = cp++;
		}
	}
	while (*IP2 == '*')
		IP2++;

	return !*IP2;
}

DLLEXPORT char*  HostIP(char* Address)
{
	LPHOSTENT he = gethostbyname(Address);

	if(he == NULL)
		return 0;
	return inet_ntoa(*(IN_ADDR*)*he->h_addr_list);
}

DLLEXPORT double SocketID(double sID)
{
	Socket* sock = (Socket*)sockets[(int)sID];
	if(sock == NULL)
		return -9;

	return sock->id;
}

DLLEXPORT double WriteByte(double Value, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	buff->writebyte((unsigned char)Value);
	return 1;
}

DLLEXPORT double WriteShort(double Value, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	buff->writeshort((short)Value);
	return 2;
}

DLLEXPORT double WriteUShort(double Value, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	buff->writeushort((unsigned short)Value);
	return 2;
}

DLLEXPORT double WriteInt(double Value, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	buff->writeint((int)Value);
	return 4;
}

DLLEXPORT double WriteUInt(double Value, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	buff->writeuint((unsigned int)Value);
	return 4;
}

DLLEXPORT double WriteFloat(double Value, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	buff->writefloat((float)Value);
	return 4;
}

DLLEXPORT double WriteDouble(double Value, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	buff->write(&Value, 8);
	return 8;
}

DLLEXPORT double WriteChars(char* Str, double bID, double Size)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	int size = (int)Size;
	if(size == 0)
		if((size = (int)strlen(Str)) == 0)
			return 0;

	buff->writechars(Str, size);
	return size;
}

DLLEXPORT double WriteString(char* Str, double bID, double Size)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	int size = (int)Size;
	if(size == 0)
		size = (int)strlen(Str);

	if(size != 0)
		buff->writechars(Str, size);
	buff->writebyte(0);
	return size + 1;
}

DLLEXPORT double CopyBuffer(double bIDDest, double bIDSrc)
{
	Buffer* buffsrc = (Buffer*)buffers[(int)bIDSrc];
	if(buffsrc == NULL)
		return -11;

	Buffer* buffdst = (Buffer*)buffers[(int)bIDDest];
	if(buffdst == NULL)
		return -11;

	int size = buffsrc->count;
	if(size != 0)
		buffdst->write(buffsrc->data, size);
	return size;
}

DLLEXPORT double CopyBufferEx(double bIDDest, double SrcStart, double SrcSize, double bIDSrc)
{
	Buffer* buffsrc = (Buffer*)buffers[(int)bIDSrc];
	if(buffsrc == NULL)
		return -11;

	Buffer* buffdst = (Buffer*)buffers[(int)bIDDest];
	if(buffdst == NULL)
		return -11;

	int start = (int)SrcStart;
	if(start >= buffsrc->count)
		return 0;
	int size = (int)SrcSize;
	if(start <= 0)
	{
		size += start;
		start = 0;
	}
	if(size <= 0)
		return 0;
	buffdst->write(buffdst->data + start, size);
	return size;
}

DLLEXPORT double ReadByte(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->readbyte();
}

DLLEXPORT double ReadShort(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->readshort();
}

DLLEXPORT double ReadUShort(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->readushort();
}

DLLEXPORT double ReadInt(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->readint();
}

DLLEXPORT double ReadUInt(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->readuint();
}

DLLEXPORT double ReadFloat(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->readfloat();
}

DLLEXPORT double ReadDouble(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->readdouble();
}

DLLEXPORT char* ReadChars(double Size, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return NULL;
	return buff->readchars((int)Size);
}

DLLEXPORT char* ReadString(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return NULL;
	return buff->readstring();
}

DLLEXPORT char* ReadSep(char* Str, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return NULL;
	return buff->readsep(Str);
}

DLLEXPORT double GetReadPos(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->readpos;
}

DLLEXPORT double GetWritePos(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->writepos;
}

DLLEXPORT double ClearBuffer(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	buff->clear();
	return 0;
}

DLLEXPORT double BufferSize(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return buff->count;
}

DLLEXPORT double SetPos(double Pos, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	
	int pos = (int)Pos;
	if(pos < 0)
		pos = 0;
	else if(pos >= buff->count)
		pos = buff->count;
	buff->readpos = pos;
	buff->writepos = pos;
	return pos;
}

DLLEXPORT double BytesLeft(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	return buff->count - buff->readpos;
}

DLLEXPORT double CreateBuffer()
{
	return buffers.Add(new Buffer());
}

DLLEXPORT double FreeBuffer(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	buffers.Remove((int)bID);
	delete(buff);
	return 0;
}

DLLEXPORT double BufferExists(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	return 1;
}

DLLEXPORT double Adler32(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	return Hash::Adler32(buff->data, buff->count);
}

DLLEXPORT char* GetMAC()
{
	static char ret[20];
	IP_ADAPTER_INFO AdapterInfo[16];
	DWORD dwBufLen = sizeof(AdapterInfo);
	DWORD dwStatus = GetAdaptersInfo(AdapterInfo, &dwBufLen);
	if(dwStatus != ERROR_SUCCESS)
		return NULL;

	PIP_ADAPTER_INFO pAdapterInfo = AdapterInfo;
	if(pAdapterInfo)
		sprintf(ret, "%02X-%02X-%02X-%02X-%02X-%02X", AdapterInfo->Address[0], AdapterInfo->Address[1],AdapterInfo->Address[2],AdapterInfo->Address[3], AdapterInfo->Address[4], AdapterInfo->Address[5]);
	else
		return NULL;
	return ret;
}

DLLEXPORT double IPToUInt(char* IP)
{
	return inet_addr(IP);
}

DLLEXPORT char* UIntToIP(double IP)
{
	IN_ADDR i;
	
	i.S_un.S_addr = (unsigned long)IP;
	return inet_ntoa(i);
}

DLLEXPORT double NetConnected()
{
	DWORD cstat;
	if(InternetGetConnectedState(&cstat, 0) == FALSE)
		return 0;
	return 1;
}

DLLEXPORT char* MD5String(char* Str)
{
	int len = (int)strlen(Str);
	if(len == 0)
		return NULL;

	return Hash::MD5(Str, len);
}

DLLEXPORT char* MD5Buffer(double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return NULL;
	if(buff->count == 0)
		return NULL;

	return Hash::MD5(buff->data, buff->count);
}

/* Provided by 39ster from the original 39dll
** This looks so freaking laggy, but here
** for backward compatibility.
*/
DLLEXPORT double RC4Crypt(char* Str, double bID)
{
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	unsigned int inplen = buff->count;
	if(inplen == 0)
		return 0;

	int keylen = min((int)strlen(Str), 256);
	if(keylen <= 0)
		return 0;

	char *inp = buff->data;
	char KeyBox[257];
	unsigned long i, j, t, x;
	char temp;

	i = j = t = 0;
	for(i = 0; i < 256; i++)
		KeyBox[i] = (char)i;
	for(i = 0; i < 256; i++)
	{
		j = (j + (unsigned long) KeyBox[i] + Str[i % keylen]) % 256;
		temp =  KeyBox[i];
		KeyBox[i] = KeyBox[j];
		KeyBox[j] =  temp;
	}
	i = j = 0;
	for(x = 0; x < inplen; x++)
	{
		i = (i + 1U) % 256;
		j = (j + (unsigned long) KeyBox[i]) % 256;
		temp = KeyBox[i];
		KeyBox[i] = KeyBox[j] ;
		KeyBox[j] = temp;
		t = ((unsigned long) KeyBox[i] + (unsigned long) KeyBox[j]) %  256 ;
		inp[x] = (inp[x] ^ KeyBox[t]);
	}
	return inplen;
}

DLLEXPORT double FileOpen(char* FileName, double Mode)
{
	File* f = new File(FileName, (int)Mode);
	if(f->hwnd == NULL)
	{
		delete(f);
		return -12;
	}
	return files.Add(f);
}

DLLEXPORT double FileClose(double fID)
{
	File* f = (File*)files[(int)fID];
	if(f == NULL)
		return -13;

	files.Remove((int)fID);
	delete(f);
	return 0;
}

DLLEXPORT double FileWrite(double fID, double bID)
{
	File* f = (File*)files[(int)fID];
	if(f == NULL)
		return -13;
	
	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;

	int start = buff->readpos;
	int size = buff->count - start;
	if(size <= 0)
		return 0;

	return f->Write(buff->data + start, size);
}

DLLEXPORT double FileRead(double fID, double Bytes, double bID)
{
	File* f = (File*)files[(int)fID];
	if(f == NULL)
		return -13;

	Buffer* buff = (Buffer*)buffers[(int)bID];
	if(buff == NULL)
		return -11;
	
	int size = (int)Bytes;
	if(size <= 0)
		return 0;

	buff->expand(size);
	int ret = f->Read(buff->data + buff->writepos, size);
	buff->writepos += ret;
	return ret;
}

DLLEXPORT double FileGetPos(double fID)
{
	File* f = (File*)files[(int)fID];
	if(f == NULL)
		return -13;

	return f->GetPos();
}

DLLEXPORT double FileSetPos(double fID, double Pos, double Rel)
{
	File* f = (File*)files[(int)fID];
	if(f == NULL)
		return -13;

	return f->SetPos((int)Pos, (bool)Rel);
}

DLLEXPORT double FileSize(double fID)
{
	File* f = (File*)files[(int)fID];
	if(f == NULL)
		return -13;

	return f->Size();
}
