/*
	On some calls, returned errors would be
		-1	Socket Error (problem creating socket)
		-2  Bind/Connect Error (problem connecting to socket)
		-3  Listen Error (problem switching socket to listen)
		-4  Trying to Close a null socket
		-5  Socket disconnected
		-6  Trying to send/recv on a listen socket
		-7  Send Error
		-8  Bad Address given
		-9  Socket does not exist
		-10 Accept returned NULL (possibly not an error)
		-11 Buffer does not exist
		-12 File could not be opened
		-13 File does not exist
*/

#pragma once
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <WinSock.h>
#include <IPTypes.h>
#include <IPHlpApi.h>
#include <WinInet.h>

#include "buffer.h"
#include "compress.h"
#include "dlist.h"
#include "file.h"
#include "hash.h"
#include "socket.h"

#define DLLEXPORT extern "C" __declspec(dllexport)

DList sockets;
DList files;
DList buffers;

/***********************************************
** ORIGINAL 39DLL FUNCTIONS (modified slightly)
***********************************************/

//TCP
DLLEXPORT double TCPConnect    (char* Address, double Port);
DLLEXPORT double TCPListen     (double Port);
DLLEXPORT double TCPAccept     (double sID);
DLLEXPORT double TCPIP         (double sID);
//DLLEXPORT double SetNagle      (double ID, double Enable); *deprecated*
DLLEXPORT double TCPConnected  (double sID);

//UDP
DLLEXPORT double UDPConnect    (double Port);

//Messages
DLLEXPORT double SendMsg       (double sID, char* Address, double Port, double bID);
DLLEXPORT double ReceiveMsg    (double sID, double Size, double bID);
DLLEXPORT double PeekMsg       (double sID, double Size, double bID);
//DLLEXPORT double SetFormat     (double sID, double Mode, char* Sep); *deprecated*

//Misc
//DLLEXPORT char*  LastInIP      (); *deprecated*
//DLLEXPORT double LastInPort    (); *deprecated*
//DLLEXPORT double SetSync       (double sID, int Mode); *deprecated*
DLLEXPORT double CloseSocket   (double sID);
DLLEXPORT double LastError     ();
DLLEXPORT char*  MyHost        ();
DLLEXPORT double CompareIP     (char* IP1, char* IP2);
//DLLEXPORT double SockStart     (); *deprecated*
//DLLEXPORT double SockExit      (); *deprecated*
DLLEXPORT char*  HostIP        (char* Address);
DLLEXPORT double SocketID      (double sID);

//Buffer Writing
DLLEXPORT double WriteByte     (double Value, double bID);
DLLEXPORT double WriteShort    (double Value, double bID);
DLLEXPORT double WriteUShort   (double Value, double bID);
DLLEXPORT double WriteInt      (double Value, double bID);
DLLEXPORT double WriteUInt     (double Value, double bID);
DLLEXPORT double WriteFloat    (double Value, double bID);
DLLEXPORT double WriteDouble   (double Value, double bID);
DLLEXPORT double WriteChars    (char* Str, double bID, double Size);
DLLEXPORT double WriteString   (char* Str, double bID, double Size);
DLLEXPORT double CopyBuffer    (double bIDDest, double bIDSrc);
DLLEXPORT double CopyBufferEx  (double bIDDest, double SrcStart, double SrcSize, double bIDSrc);

//Buffer Reading
DLLEXPORT double ReadByte      (double bID);
DLLEXPORT double ReadShort     (double bID);
DLLEXPORT double ReadUShort    (double bID);
DLLEXPORT double ReadInt       (double bID);
DLLEXPORT double ReadUInt      (double bID);
DLLEXPORT double ReadFloat     (double bID);
DLLEXPORT double ReadDouble    (double bID);
DLLEXPORT char*  ReadChars     (double Size, double bID);
DLLEXPORT char*  ReadString    (double bID);
DLLEXPORT char*  ReadSep       (char* Str, double bID);

//Buffer Misc
DLLEXPORT double GetReadPos    (double bID);
DLLEXPORT double GetWritePos   (double bID);
DLLEXPORT double ClearBuffer   (double bID);
DLLEXPORT double BufferSize    (double bID);
DLLEXPORT double SetPos        (double Pos, double bID);
DLLEXPORT double BytesLeft     (double bID);

//Multiple Buffers
DLLEXPORT double CreateBuffer  ();
DLLEXPORT double FreeBuffer    (double bID);
DLLEXPORT double BufferExists  (double bID);

//Utilities
DLLEXPORT double Adler32       (double bID);
DLLEXPORT char*  GetMAC        ();
DLLEXPORT double IPToUInt      (char* IP);
DLLEXPORT char*  UIntToIP      (double IP);
DLLEXPORT double NetConnected  ();

//MD5
DLLEXPORT char*  MD5String     (char* Str);
DLLEXPORT char*  MD5Buffer     (double bID);

//Encryption
DLLEXPORT double RC4Crypt      (char* Str, double bID);

//File
DLLEXPORT double FileOpen      (char* FileName, double Mode);
DLLEXPORT double FileClose     (double fID);
DLLEXPORT double FileWrite     (double fID, double bID);
DLLEXPORT double FileRead      (double fID, double Bytes, double bID);
DLLEXPORT double FileGetPos    (double fID);
DLLEXPORT double FileSetPos    (double fID, double Pos, double Rel);
DLLEXPORT double FileSize      (double fID);

/***********************************************
**        Extended Functionality
***********************************************/


