#include "socket.h"

int Socket::SIN_size;
char* Socket::dbuff;
unsigned long Socket::sync;

void Socket::Init()
{	
	WSADATA wsaData;
	WSAStartup(MAKEWORD(1,1), &wsaData);
	dbuff = new char[65536];
	SIN_size = sizeof(SOCKADDR_IN);
	sync = 1;
}

void Socket::Clean()
{
	WSACleanup();
	delete(dbuff);
}

unsigned long Socket::IP(char* address)
{
	HOSTENT* he = gethostbyname(address);

	if(he == NULL)
		return 0;
	return *(unsigned long*)he->h_addr_list;
}

Socket::Socket()
{
	id = 0;
	options = 0;
	bin = new Buffer();
}

Socket::~Socket()
{
	if(id != 0)
		closesocket(id);
	delete(bin);
}

int Socket::Listen(int typ, unsigned short port)
{
	type = typ;

	SOCKADDR_IN ad;
	
	ad.sin_family = AF_INET;
	ad.sin_addr.s_addr = INADDR_ANY;
	ad.sin_port = htons(port);

	switch (typ)
	{
		case 0: //IPv4 UDP
			if ((id = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == INVALID_SOCKET)
			{
				id = 0;
				return -1;
			}
			ioctlsocket(id, FIONBIO, &sync);
			if (bind(id, (SOCKADDR*)&ad, SIN_size) == SOCKET_ERROR)
			{
				closesocket(id);
				id = 0;
				return -2;
			}
			return 0;

		case 1: //IPv4 TCP
			if ((id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == INVALID_SOCKET)
			{
				id = 0;
				return -1;
			}
			ioctlsocket(id, FIONBIO, &sync);
			if(bind(id, (SOCKADDR*)&ad, SIN_size) == SOCKET_ERROR)
			{
				closesocket(id);
				id = 0;
				return -2;
			}
			if(listen(id, 16) == SOCKET_ERROR)
			{
				closesocket(id);
				id = 0;
				return -3;
			}
			options = 1;
			return 0;
	}
	return -1;
}

Socket* Socket::Accept()
{
	if(id == 0)
		return NULL;
	switch(type)
	{
		case 1: //IPv4 TCP
			SOCKET child = accept(id, (SOCKADDR*)&other, &SIN_size);

			if(child == INVALID_SOCKET)
				return NULL;

			Socket* schild = new Socket();

			schild->type = 1;
			schild->options = 0;
			schild->id = child;
			ioctlsocket(child, FIONBIO, &sync);
			return schild;
	}
	return NULL;
}

int Socket::Connect(unsigned long ip, unsigned short port, int typ)
{
	type = typ;
	switch(typ)
	{
		case 1: //IPv4 TCP
			if((id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == SOCKET_ERROR)
			{
				id = 0;
				return -1;
			}
			other.sin_family = AF_INET;
			other.sin_addr = *(IN_ADDR*)&ip;
			other.sin_port = htons(port);
			ioctlsocket(id, FIONBIO, &sync);
			if(connect(id, (SOCKADDR*)&other, SIN_size) == SOCKET_ERROR)
			{
				closesocket(id);
				id = 0;
				return -2;
			}
			return 0;
	}
	return -1;
}

int Socket::Send(Buffer* b)
{
	if(id == 0)
		return -5;
	if(options & 1)
		return -6;

	Buffer* out;
	int size;

	switch(options & 112)
	{
		case 0: //raw stream output format
			size = (b->count);
			if(size < 1)
				return 0;
			out = b;
			break;
	}
	switch(type)
	{
		case 0: //IPv4 UDP
			if(size > 8192)
				size = 8192;
			size = sendto(id, out->data, size, 0, (SOCKADDR*)&other, SIN_size);
			if(size == SOCKET_ERROR)
				return -7;
			return size;

		case 1: //IPv4 TCP
			size = send(id, out->data, size, 0);
			if(size == SOCKET_ERROR)
				return -7;
			return size;
	}
	return -1;
}

int Socket::Recv(Buffer* b)
{
	if(id == 0)
		return -5;
	if(options & 1)
		return -6;

	int size;

	switch(type)
	{
		case 0: //IPv4 UDP
			size = recvfrom(id, dbuff, 65536, 0, (SOCKADDR*)&other, &SIN_size);
			break;

		case 1: //IPv4 TCP
			size = recv(id, dbuff, 65536, 0);
			break;
	}
	if(size > 0)
	{
		switch(options & 14)
		{
			case 0: //raw stream input format option
				b->write(dbuff, size);
				return size;
		}
	}
	return 0;
}

int Socket::Peek(Buffer* b)
{
	if(id == 0)
		return -5;
	if(options & 1)
		return -6;

	int size;

	switch(type)
	{
		case 0: //IPv4 UDP
			size = recvfrom(id, dbuff, 65536, 0, (SOCKADDR*)&other, &SIN_size);
			break;

		case 1: //IPv4 TCP
			size = recv(id, dbuff, 65536, 0);
			break;
	}
	if(size > 0)
	{
		//this is always raw stream format return
		b->write(dbuff, size);
		return size;
	}
	return 0;
}

unsigned long Socket::GetIP()
{
	if(id == 0)
		return 0;
	return *(unsigned long*)&other.sin_addr;
}

unsigned short Socket::GetPort()
{
	if(id == 0)
		return 0;
	return other.sin_port;
}

bool Socket::Connected()
{
	if(id == 0)
		return false;
	switch(type)
	{
		case 1: //IPv4 TCP
			char b;

			if(recv(id, &b, 1, MSG_PEEK) == SOCKET_ERROR)
				if(WSAGetLastError() != WSAEWOULDBLOCK)
					return false;
			return true;

		default:
			return false;
	}
}